/*
 *         _        ___
 *    /\ / /___    / _ \ /\ /\  _ ___
 *   / // // _ \  / // // // // // _ \
 *  / // // // / / ___// // // // // /
 *  \___//_//_/ /_/   /_/ \___/ \_  /
 *                             \___/
 * 
 *  Compunach UnPlug
 *  Copyright (C) 2009 David Batley <unplug@dbatley.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

UnPlug2SearchPage = {
	UnPlug2SearchPage : function (args) {
		// parent window (the one we want to search)
		this._win = args.tgt_window;
		
		// parent window's getBrowser()
		this._gbrowser = args.gbrowser;
		
		// firefox 
		this._download_mgr = Components.classes["@mozilla.org/download-manager;1"]
			.getService(Components.interfaces.nsIDownloadManager);
		
		// io service
		this._io_service = Components.classes["@mozilla.org/network/io-service;1"]
			.getService(Components.interfaces.nsIIOService)
		
		// capabilities - what can we do
		this.capabilities = {};
		this.capabilities['downthemall'] = (
			UnPlug2.get_root_pref("extensions.{DDC359D1-844A-42a7-9AA1-88A850A938A8}.description")
			? true : false )
		this.capabilities['opener'] = UnPlug2.get_pref("opener");
		
		// set this to true to stop everything
		this._stopped = false;
		
		UnPlug2.log("Capabilities: " + this.capabilities.toSource())
	},
	
	/**
	 * search the parent window for media, returning search result objects
	 */
	do_search : function () {
		document.getElementById("dynamic_download").value = UnPlug2.str("search_busy");
		// TODO - localize
		document.getElementById("dynamic_results").value = "No results yet";
		
		try {
			UnPlug2Search._reset(); // this may not be needed because we always start from a fresh window
			UnPlug2Search.search(this._win, this._search_callback)
			this.poll()
		} catch (e) {
			UnPlug2.log(e);
		}
	},
	
	/**
	 * See if we've finished looking for suff yet
	 */
	poll : function () {
		try {
			var statusinfo = UnPlug2Search.statusinfo();
			document.getElementById("dynamic_download").value = statusinfo.text;
			if (statusinfo.finished) {
				with (document.getElementById("search_progress")) {
					mode = "determined";
					value = "100";
				}
				var all_results = document.getElementsByTagName("unplug_result");
				// TODO - localize
				document.getElementById("stop_button").disabled = true;
				if (all_results.length == 0) {
					document.getElementById("dynamic_results").value = "No results";
					document.getElementById("dynamic_results").className = "failed";
				} else if (all_results.length == 1) {
					document.getElementById("dynamic_results").value = "1 result";
				} else {
					document.getElementById("dynamic_results").value = "# results".replace("#", all_results.length);
				}
			} else {
				with (document.getElementById("search_progress")) {
					mode = "undetermined";
				}
				window.setTimeout(UnPlug2SearchPage.poll, 500);
			}
		} catch (e) {
			UnPlug2.log(e);
		}
	},
	
	// drag+drop observer
	drag_and_drop_observer : {
		onDragStart: function (e, transferData, action) {
			var url = e.target.getAttribute("url");
			transferData.data = new TransferData();
			transferData.data.addDataForFlavour("text/unicode", url);
			/* transferData.data.addDataForFlavour("text/html", url); */
		}
	},
	
	/**
	 * Callback for UnPlug2Rules.search
	 * Called for each result found. This may be asynchromous (ie, after additional files are downloaded).
	 */
	_search_callback : function (result) {
		UnPlug2.log("FOUND: " + result.url + " from " + result.trace);
		try {
			var reselem = document.createElement("unplug_result");
			reselem.setAttribute("url", result.url);
			reselem.setAttribute("type", result.type || "");
			reselem.setAttribute("title", result.title || "No title");
			reselem.setAttribute("description", result.description || "");
			reselem.setAttribute("thumbnail", result.thumbnailurl || "");
			reselem.setAttribute("url_host", result.host || "");
			reselem.setAttribute("url_protocol", result.protocol || "");
			
			if (result.is_link()) {
				reselem.addEventListener("draggesture", function (e) {
					nsDragAndDrop.startDrag(e, UnPlug2SearchPage.drag_and_drop_observer);
					}, false);
			}

			document.getElementById("results").appendChild(reselem);
			reselem.setDownloaders(UnPlug2SearchPage.capabilities, result);
			reselem.setOpeners(UnPlug2SearchPage.capabilities, result);
			
			var duplicate_elem = reselem.checkDuplicates(reselem.parentNode);
			if (duplicate_elem) {
				UnPlug2.log("Duplicate element");
				if (!UnPlug2.get_pref("show_duplicates", false)) {
					var things_to_copy = ["thumbnail", "description"];
					for (var j=0; j < things_to_copy.length; j++) {
						duplicate_elem.setAttribute(things_to_copy[j], duplicate_elem.getAttribute(things_to_copy[j]) || reselem.getAttribute(things_to_copy[j]));
					}
					reselem.parentNode.removeChild(reselem);
				}
			}
		} catch(e) {
			UnPlug2.log("ERROR displaying result " + e);
		}
	},
	
	toString : function () {
		return '<UnPlug2SearchPage>';
	},
	
	open_new_window : function (reselem) {
		window.open(reselem.getAttribute("url"));
	},
	
	open_new_tab : function (reselem) {
		var t = UnPlug2SearchPage._gbrowser.addTab(reselem.getAttribute("url"));
		UnPlug2SearchPage._gbrowser.selectedTab = t;
	},
	
	open_over_window : function (reselem) {
		UnPlug2SearchPage._win.location = reselem.getAttribute("url");
	},
	
	download_saveas : function (reselem) {
		var source_url = reselem.getAttribute("url");
		var dest = "/home/user/DELETEME";
		var file = UnPlug2SearchPage._save_as_box();
		if (!file)
			return;
		UnPlug2SearchPage._download_with_downloadmgr(source_url, file);
	},
	
	/**
	 * return file or null.
	 */
	_save_as_box : function () {
		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var filepicker = Components.classes["@mozilla.org/filepicker;1"]
			.createInstance(nsIFilePicker);
		filepicker.init(window, "Save as", nsIFilePicker.modeSave);
		var ret = filepicker.show();
		
		if (ret != nsIFilePicker.returnOK && ret != nsIFilePicker.returnReplace)
			return null; // cancelled
		
		return filepicker.file;
	},
	
	/*
	 * save String url into a nsIFile file
	 */
	_download_with_downloadmgr : function (source_url, file) {
		var persist = Components.classes["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"]
			.createInstance(Components.interfaces.nsIWebBrowserPersist);
		
		var ioFile = this._io_service.newFileURI(file);   
		var obj_URI = this._io_service.newURI(source_url, null, null);
		
		persist.progressListener = this._download_mgr.addDownload(
			0, // aDownloadType
			obj_URI, // source
			ioFile, // target
			"", // aDisplayName
			null, //aMIMEInfo
			Date.now(), // aStartTim
			null, // aTempFile,
			persist); // aCancelable
		
		persist.saveURI(obj_URI, null, null, null, "", ioFile);
	},
	
	download_dta : function (reselem) {
		var source_url = reselem.getAttribute("url");
		window.opener.DTA_AddingFunctions.saveSingleLink(
			false, //turbo
			source_url, //url
			String(UnPlug2SearchPage._win.location), // referer
			null, // description
			null) // post data
	},
	
	send_nothing_found_msg : function () {
		if (!confirm(UnPlug2.str("nothing_found_send_data")))
			return;
		UnPlug2SearchPage.send_nothing_found_msg_noask();
	},
	
	send_nothing_found_msg_noask : function () {
		var post_data = "url=" + escape(UnPlug2SearchPage._win.location) + "&version=" + escape(UnPlug2.version) + "&revision=" + escape(UnPlug2.revision);
		
		var el = document.getElementById("notfound_button");
		if (!el) {
			UnPlug2.log("No element in xul called notfound");
			return;
		}
		el.disabled = "true";
		el.label = UnPlug2.str("nothing_found_sending");
		
		var dl = new UnPlug2Download(
			null, // id
			"http://unplug.dbatley.com/ajax/notfound.php",
			post_data,
			UnPlug2SearchPage.done_nothing_found_msg,
			UnPlug2SearchPage.failed_nothing_found_msg,
			60000)
	},
	
	done_nothing_found_msg : function () {
		var el = document.getElementById("notfound_button");
		el.label = UnPlug2.str("nothing_found_done");
	},
	
	failed_nothing_found_msg : function () {
		var el = document.getElementById("notfound_button");
		el.label = UnPlug2.str("nothing_found_failed");
		el.disabled = "false";
	},
	
	/*
	 * Stop downloading/searching pages!
	 */
	abort : function () {
		UnPlug2Search._stopped = true;
		UnPlug2Search.abort();
	},
	
	/**
	 * When someone clicks the configure button on unplug_result
	 */
	configure : function (tabname) {
		window.openDialog("chrome://unplug/content/config.xul", "", "", tabname);
	},
	
	endofobject : 1 }

//init
UnPlug2SearchPage.UnPlug2SearchPage(window.arguments[0])



